@@ -1,8 +1,6 @@
 package Module::Build::Bundle;
 
-# $Id: Bundle.pm 8100 2013-03-15 08:23:53Z jonasbn $
-
-use 5.006;    #$^V
+use 5.008;    #$^V
 use strict;
 use warnings;
 use Carp qw(croak);
@@ -10,15 +8,12 @@ use Cwd qw(getcwd);
 use Tie::IxHash;
 use English qw( -no_match_vars );
 use File::Slurp;    #read_file
-use base qw(Module::Build);
+use base qw(Module::Build::Base);
 use utf8;
 
 use constant EXTENDED_POD_LINK_VERSION => 5.12.0;
-use constant PERMISSION_MASK           => '07777';
-use constant WRITEPERMISSION           => '0644';
-use constant FILEMODE                  => 2;
 
-our $VERSION = '0.12';
+our $VERSION = '0.14';
 
 #HACK: we need a writable copy for testing purposes
 ## no critic qw(Variables::ProhibitPackageVars Variables::ProhibitPunctuationVars)
@@ -79,12 +74,12 @@ sub ACTION_contents {
         || $self->{properties}->{module_name};
 
     #HACK: induced from test suite
-    my $dir = $self->notes('temp_wd') ? $self->notes('temp_wd') : 'blib/lib';
+    my $dir = $self->notes('temp_wd') ? $self->notes('temp_wd') : $cwd .'/t/';
 
     ## no critic qw(ValuesAndExpressions::ProhibitNoisyQuotes)
-    my $file = ( join '/', ( $cwd, $dir, @path ) ) . '.pm';
+    my $file = ( join '/', ( $dir, @path ) ) . '.pm';
 
-    my $contents = read_file($file);
+    my $contents = read_file($file) or croak "Unable to read file: $file - $!";
 
     my $rv = $contents =~ s/=head1\s*$section_header\s*.*=head1/$pod/s;
 
@@ -92,148 +87,266 @@ sub ACTION_contents {
         croak "No $section_header section replaced";
     }
 
-    ## no critic (Bangs::ProhibitBitwiseOperators)
-    my $permissions = ( stat $file )[FILEMODE] & PERMISSION_MASK;
-    chmod WRITEPERMISSION, $file
-        or croak "Unable to make file: $file writable - $!";
-
     open my $fout, '>', $file
         or croak "Unable to open file: $file - $!";
 
     print $fout $contents;
 
-    chmod $permissions, $file
-        or croak "Unable to reinstate permissions for file: $file - $!";
-
     close $fout or croak "Unable to close file: $file - $!";
 
     return 1;
 }
 
-#lifted from Module::Build::Base
-sub create_mymeta {
-    my ($self)     = @_;
-    my $mymetafile = $self->mymetafile;
-    my $metafile   = $self->metafile;
-
-    # cleanup
-    if ( $self->delete_filetree($mymetafile) ) {
-        $self->log_verbose("Removed previous '$mymetafile'\n");
-    }
-    $self->log_info(
-        "Creating new '$mymetafile' with configuration results\n");
-
-    # use old meta and update prereqs, if possible
-    my $mymeta;
-    if ( -e $metafile ) {
-        $mymeta = eval { $self->read_metafile( $self->metafile ) };
-    }
-
-    # if we read META OK, just update it
-    if ( defined $mymeta ) {
-        my $prereqs = $self->_normalize_prereqs;
-        for my $t ( keys %{$prereqs} ) {
-            $mymeta->{$t} = $prereqs->{$t};
-        }
-    }
-
-    # but generate from scratch, ignoring errors if META doesn't exist
-    else {
-        $mymeta = $self->get_metadata( fatal => 0 );
-    }
-
-    my $package = ref $self;
-
-    # MYMETA is always static
-    $mymeta->{dynamic_config} = 0;
-
-    # Note which M::B created it
-    #JONASBN: changed from originally lifted code
-    $mymeta->{generated_by} = "$package version $VERSION";
-
-    $self->write_metafile( $mymetafile, $mymeta );
-    return 1;
-}
-
-#lifted from Module::Build::Base
-sub get_metadata {
-    my ( $self, %args ) = @_;
-
-    my $metadata = {};
-    $self->prepare_metadata( $metadata, undef, \%args );
-
-    my $package = ref $self;
-
-    #JONASBN: changed from originally lifted code
-    $metadata->{generated_by} = "$package version $VERSION";
-
-    #JONASBN: changed from originally lifted code
-    $metadata->{configure_requires} = { "$package" => $VERSION };
-
-    return $metadata;
-}
-
-#Lifed from Module::Build::Base
+#Lifted from Module::Build::Base
 sub do_create_metafile {
-    my $self = shift;
-    return if $self->{wrote_metadata};
+  my $self = shift;
+  return if $self->{wrote_metadata};
+  my $p = $self->{properties};
 
-    my $p = $self->{properties};
+  unless ($p->{license}) {
+    $self->log_warn("No license specified, setting license = 'unknown'\n");
+    $p->{license} = 'unknown';
+  }
 
-    #JONASBN: changed from originally lifted code
-    my $metafile = $self->metafile;
+  my @metafiles = ( $self->metafile, $self->metafile2 );
+  # If we're in the distdir, the metafile may exist and be non-writable.
+  $self->delete_filetree($_) for @metafiles;
 
-    unless ( $p->{license} ) {
-        $self->log_warn(
-            "No license specified, setting license = 'unknown'\n");
-        $p->{license} = 'unknown';
+  # Since we're building ourself, we have to do some special stuff
+  # here: the ConfigData module is found in blib/lib.
+  local @INC = @INC;
+  if (($self->module_name || '') eq 'Module::Build') {
+    $self->depends_on('config_data');
+    push @INC, File::Spec->catdir($self->blib, 'lib');
+  }
+
+  my $meta_obj = $self->_get_meta_object(
+    quiet => 0, fatal => 1, auto => 1
+  );
+
+  #JONASBN: Changed file parameter
+  my @created = $self->_write_meta_files( $meta_obj, $self->metafile );
+
+  if ( @created ) {
+    $self->{wrote_metadata} = 1;
+    $self->_add_to_manifest('MANIFEST', $_) for @created;
+  }
+  return 1;
+}
+
+#lifted from Module::Build::Base, sets generated_by 
+sub get_metadata {
+  my ($self, %args) = @_;
+
+  my $fatal = $args{fatal} || 0;
+  my $p = $self->{properties};
+
+  $self->auto_config_requires if $args{auto};
+
+  # validate required fields
+  foreach my $f (qw(dist_name dist_version dist_author dist_abstract license)) {
+    my $field = $self->$f();
+    unless ( defined $field and length $field ) {
+      my $err = "ERROR: Missing required field '$f' for metafile\n";
+      if ( $fatal ) {
+        die $err;
+      }
+      else {
+        $self->log_warn($err);
+      }
     }
-    unless ( exists $self->valid_licenses->{ $p->{license} } ) {
-        croak "Unknown license type '$p->{license}'";
+  }
+
+  my %metadata = (
+    name => $self->dist_name,
+    version => $self->normalize_version($self->dist_version),
+    author => $self->dist_author,
+    abstract => $self->dist_abstract,
+    #JONASBN changed the generated_by
+    generated_by => "Module::Build::Bundle version $Module::Build::Bundle::VERSION",
+    'meta-spec' => {
+      version => '2',
+      url     => 'http://search.cpan.org/perldoc?CPAN::Meta::Spec',
+    },
+    dynamic_config => exists $p->{dynamic_config} ? $p->{dynamic_config} : 1,
+    release_status => $self->release_status,
+  );
+
+  my ($meta_license, $meta_license_url) = $self->_get_license;
+  $metadata{license} = [ $meta_license ];
+  $metadata{resources}{license} = [ $meta_license_url ] if defined $meta_license_url;
+
+  $metadata{prereqs} = $self->_normalize_prereqs;
+
+  if (exists $p->{no_index}) {
+    $metadata{no_index} = $p->{no_index};
+  } elsif (my $pkgs = eval { $self->find_dist_packages }) {
+    $metadata{provides} = $pkgs if %$pkgs;
+  } else {
+    $self->log_warn("$@\nWARNING: Possible missing or corrupt 'MANIFEST' file.\n" .
+                    "Nothing to enter for 'provides' field in metafile.\n");
+  }
+
+  if (my $add = $self->meta_add) {
+    if (not exists $add->{'meta-spec'} or $add->{'meta-spec'}{version} != 2) {
+      require CPAN::Meta::Converter;
+      if (CPAN::Meta::Converter->VERSION('2.141170')) {
+        $add = CPAN::Meta::Converter->new($add)->upgrade_fragment;
+        delete $add->{prereqs}; # XXX this would now overwrite all prereqs
+      }
+      else {
+        $self->log_warn("Can't meta_add without CPAN::Meta 2.141170");
+      }
     }
 
-    # If we're in the distdir, the metafile may exist and be non-writable.
-    $self->delete_filetree($metafile);
-    $self->log_info("Creating $metafile\n");
-
-    # Since we're building ourself, we have to do some special stuff
-    # here: the ConfigData module is found in blib/lib.
-    local @INC = @INC;
-    if ( ( $self->module_name || '' ) eq 'Module::Build' ) {
-        $self->depends_on('config_data');
-        push @INC, File::Spec->catdir( $self->blib, 'lib' );
+    while (my($k, $v) = each %{$add}) {
+      $metadata{$k} = $v;
     }
+  }
 
-    if ($self->write_metafile(
-            $self->metafile, $self->get_metadata( fatal => 1 ),
-        )
-        )
-    {
-        $self->{wrote_metadata} = 1;
-        $self->_add_to_manifest( 'MANIFEST', $metafile );
+  if (my $merge = $self->meta_merge) {
+    if (eval { require CPAN::Meta::Merge }) {
+      %metadata = %{ CPAN::Meta::Merge->new(default_version => '1.4')->merge(\%metadata, $merge) };
     }
+    else {
+      $self->log_warn("Can't merge without CPAN::Meta::Merge");
+    }
+  }
 
-    return 1;
+  return \%metadata;
+}
+
+#lifted from Module::Build::Base, added package and version resolution and 
+# addition to configure requires
+sub prepare_metadata {
+  my ($self, $node, $keys) = @_;
+  my $p = $self->{properties};
+
+  #JONASBN: Added package resolution
+  my $package = ref $self;
+  my $version = $package::VERSION;
+
+  # A little helper sub
+  my $add_node = sub {
+    my ($name, $val) = @_;
+    $node->{$name} = $val;
+    push @$keys, $name if $keys;
+  };
+
+  foreach (qw(dist_name dist_version dist_author dist_abstract license)) {
+    (my $name = $_) =~ s/^dist_//;
+    $add_node->($name, $self->$_());
+    die "ERROR: Missing required field '$_' for META.yml\n"
+      unless defined($node->{$name}) && length($node->{$name});
+  }
+  $node->{version} = $self->normalize_version($node->{version}); 
+
+  if (defined( my $l = $self->license )) {
+    die "Unknown license string '$l'"
+      unless exists $self->valid_licenses->{ $l };
+
+    if (my $key = $self->valid_licenses->{ $l }) {
+      my $class = "Software::License::$key";
+      if (eval "use $class; 1") {
+        # S::L requires a 'holder' key
+        $node->{resources}{license} = $class->new({holder=>"nobody"})->url;
+      }
+      else {
+        $node->{resources}{license} = $self->_license_url($l);
+      }
+    }
+    # XXX we are silently omitting the url for any unknown license
+  }
+
+  # copy prereq data structures so we can modify them before writing to META
+  my %prereq_types;
+  for my $type ( 'configure_requires', @{$self->prereq_action_types} ) {
+    if (exists $p->{$type}) {  
+      for my $mod ( keys %{ $p->{$type} } ) {
+        $prereq_types{$type}{$mod} = 
+          $self->normalize_version($p->{$type}{$mod});
+      }
+    }
+  }
+
+  # add current Module::Build to configure_requires if there 
+  # isn't one already specified (but not ourself, so we're not circular)
+  if ( $self->dist_name ne 'Module-Build' 
+    && $self->auto_configure_requires
+    && ! exists $prereq_types{'configure_requires'}{'Module::Build'}
+  ) {
+    $prereq_types{configure_requires}{'Module::Build'} = $VERSION;
+    #JONASBN added configure requires
+    $prereq_types{configure_requires}{$package} = $version;
+  }
+
+  for my $t ( keys %prereq_types ) {
+      $add_node->($t, $prereq_types{$t});
+  }
+
+  if (exists $p->{dynamic_config}) {
+    $add_node->('dynamic_config', $p->{dynamic_config});
+  }
+  my $pkgs = eval { $self->find_dist_packages };
+  if ($@) {
+    $self->log_warn("$@\nWARNING: Possible missing or corrupt 'MANIFEST' file.\n" .
+        "Nothing to enter for 'provides' field in META.yml\n");
+  } else {
+    $node->{provides} = $pkgs if %$pkgs;
+  }
+;
+  if (exists $p->{no_index}) {
+    $add_node->('no_index', $p->{no_index});
+  }
+
+  $add_node->('generated_by', "$package version $version");
+
+  $add_node->('meta-spec', 
+        {version => '1.4',
+         url     => 'http://module-build.sourceforge.net/META-spec-v1.4.html',
+        });
+
+  while (my($k, $v) = each %{$self->meta_add}) {
+    $add_node->($k, $v);
+  }
+
+  while (my($k, $v) = each %{$self->meta_merge}) {
+    $self->_hash_merge($node, $k, $v);
+  }
+
+  return $node;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding utf8
+
+=begin markdown
+
+[![CPAN version](https://badge.fury.io/pl/Module-Build-Bundle.svg)](http://badge.fury.io/pl/Module-Build-Bundle)
+[![Build Status](https://travis-ci.org/jonasbn/mbb.svg?branch=master)](https://travis-ci.org/jonasbn/mbb)
+[![Coverage Status](https://coveralls.io/repos/jonasbn/mbb/badge.png)](https://coveralls.io/r/jonasbn/mbb)
+
+=end markdown
+
 =head1 NAME
 
 Module::Build::Bundle - subclass for supporting Tasks and Bundles
 
 =head1 VERSION
 
-This documentation describes version 0.11
+This documentation describes version 0.14
 
 =head1 SYNOPSIS
 
     #In your Build.PL
     use Module::Build::Bundle;
-    
-    #Example lifted from: Perl::Critic::logicLAB 
+
+    #Example lifted from: Perl::Critic::logicLAB
     my $build = Module::Build::Bundle->new(
         dist_author   => 'Jonas B. Nielsen (jonasbn), <jonasbn@cpan.org>',
         module_name   => 'Perl::Critic::logicLAB',
@@ -244,16 +357,16 @@ This documentation describes version 0.11
             'Perl::Critic::Policy::logicLAB::RequireVersionFormat' => '0',
         },
     );
-    
+
     $build->create_build_script();
 
 
     #In your shell
     % ./Build contents
-    
+
     #Or implicitly executing contents action
     % ./Build
-    
+
 =head1 DESCRIPTION
 
 =head2 FEATURES
@@ -264,7 +377,7 @@ This documentation describes version 0.11
 
 =item * Links to required/listed distributions, with or without versions
 
-=item * Links to specific versions of distributions for perl 5.12.0 or newer if a 
+=item * Links to specific versions of distributions for perl 5.12.0 or newer if a
 version is specified
 
 =item * Inserts a POD section named CONTENTS or something specified by the
@@ -287,7 +400,7 @@ By default it overwrites the CONTENTS section with a POD link listing. You can
 specify a note indicating if what section you want to overwrite using the
 section_header note.
 
-    #Example lifted from: Perl::Critic::logicLAB 
+    #Example lifted from: Perl::Critic::logicLAB
     my $build = Module::Build::Bundle->new(
         dist_author   => 'Jonas B. Nielsen (jonasbn), <jonasbn@cpan.org>',
         module_name   => 'Perl::Critic::logicLAB',
@@ -298,7 +411,7 @@ section_header note.
             'Perl::Critic::Policy::logicLAB::RequireVersionFormat' => '0',
         },
     );
-    
+
     $build->notes('section_header' => 'POLICIES');
 
     $build->create_build_script();
@@ -337,25 +450,6 @@ This is a simple wrapper around the standard action: L<Module::Build|Module::Bui
 build action. It checks whether L</ACTION_contents> have been executed, if not
 it executes it.
 
-=head2 create_mymeta
-
-This method has been lifted from L<Module::Build::Base|Module::Build::Base> and altered.
-
-It sets the:
-
-=over
-
-=item * 'generated by <package> version <package version>' string in F<MYMETA.yml>
-
-=back
-
-For Module::Build::Bundle:
-
-    #Example MYMETA.yml
-    configure_requires:
-        Module::Build::Bundle: 0.01
-    generated_by: 'Module::Build::Bundle version 0.01'
-    
 =head2 get_metadata
 
 This method has been lifted from L<Module::Build::Base|Module::Build::Base> and
@@ -367,8 +461,6 @@ It sets:
 
 =item * 'generated by <package> version <package version>' string in F<META.yml>
 
-=item * configure_requires: <package>: <version> 
-
 =back
 
 For Module::Build::Bundle:
@@ -378,6 +470,19 @@ For Module::Build::Bundle:
         Module::Build::Bundle: 0.01
     generated_by: 'Module::Build::Bundle version 0.01'
 
+=head2 prepare_metadata
+
+This method has been lifted from L<Module::Build::Base|Module::Build::Base> and
+altered.
+
+It sets:
+
+=over
+
+=item * configure_requires: <package>: <version>
+
+=back
+
 =head2 do_create_metafile
 
 This method has been lifted from L<Module::Build::Base|Module::Build::Base> and
@@ -386,6 +491,8 @@ altered.
 The method was overwritten to be more testable. The method created the relevant
 META file.
 
+It passes a more general file parameter for testing instead of a hard-coded filename.
+
 =head1 DIAGNOSTICS
 
 =over
@@ -398,19 +505,21 @@ will die with the above message.
 The default minimal section should look something like:
 
     =head1 CONTENTS
-    
+
     =head1
-    
+
 Or if you provide your own section_header
 
     =head1 <section header>
-    
+
     =head1
 
 =back
 
 =head1 CONFIGURATION AND ENVIRONMENT
 
+The only special configuration necessary is described in the section below.
+
 =head2 CONTENTS SECTION
 
 The module does per default look for the section named: CONTENTS.
@@ -419,7 +528,7 @@ This is the section used in Bundles, this can be overwritten using the section
 parameter.
 
 For example L<Perl::Critic::logicLAB|Perl::Critic::logicLAB> uses a section
-named POLICIES and L<Task::BeLike::JONASBN> uses DEPENDENCIES.
+named POLICIES and L<Task::BeLike::JONASBN> uses DEPENDENCIES as section header.
 
 The problem is that the section has to be present or else the contents action
 will throw an error.
@@ -431,7 +540,7 @@ however no longer recommended and L<Task> provides a better way.
 
 =over
 
-=item * perl 5.6.0
+=item * perl 5.8.0
 
 =item * L<Module::Build::Base|Module::Build::Base>, part of the L<Module::Build>
 distribution
@@ -440,7 +549,8 @@ distribution
 
 =head1 INCOMPATIBILITIES
 
-The distribution requires perl version from 5.6.0 and up.
+The distribution requires perl version from 5.8.0 and up, earlier versions of perl
+are not supported.
 
 =head1 BUGS AND LIMITATIONS
 
@@ -471,9 +581,11 @@ Please report any bugs or feature requests via:
 
 =over
 
+=item * Github issues: L<https://github.com/jonasbn/mbb/issues>
+
 =item * email: bug−module-build-bundle at rt.cpan.org
 
-=item * HTTP: L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Module-Build-Bundle>
+=item * RT: L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Module-Build-Bundle>
 
 =back
 
@@ -484,11 +596,11 @@ Please report any bugs or feature requests via:
     ---------------------------- ------ ------ ------ ------ ------ ------ ------
     File                           stmt   bran   cond    sub    pod   time  total
     ---------------------------- ------ ------ ------ ------ ------ ------ ------
-    lib/Module/Build/Bundle.pm     48.5   13.9   14.3   84.2  100.0  100.0   45.7
-    Total                          48.5   13.9   14.3   84.2  100.0  100.0   45.7
+    lib/Module/Build/Bundle.pm     44.5   18.3   23.0   70.5  100.0  100.0   39.6
+    Total                          44.5   18.3   23.0   70.5  100.0  100.0   39.6
     ---------------------------- ------ ------ ------ ------ ------ ------ ------
 
-The above coverage report is based on release 0.11
+The above coverage report is based on release 0.13
 
 =head1 QUALITY AND CODING STANDARD
 
@@ -496,7 +608,7 @@ The code passes L<Perl::Critic> tests a severity: 1 (brutal)
 
 The following policies have been disabled:
 
-       Perl::Critic::Policy::InputOutput::RequireBracedFileHandleWithPrint
+      L<Perl::Critic::Policy::InputOutput::RequireBracedFileHandleWithPrint>
 
 L<Perl::Critic> resource file, can be located in the F<t/> directory of the
 distribution see F <t/perlcriticrc>
@@ -511,8 +623,20 @@ L<Perl::Tidy> resource file, can be obtained from the following URL:
 
 =head1 DEVELOPMENT
 
+The Module::Build::Bundle repository is public available on Github, pull requests most
+welcome.
+
 =head1 TODO
 
+=over
+
+=item * Exchange the fragile POD handling with Pod::Weaver
+
+=item * Methods lifted from L<Module::Build::Base>, could be backported to L<Module::Build::Base>, 
+as patches if these can be implemented in a acceptable state.
+
+=back
+
 Please see: L<https://logiclab.jira.com/browse/MBB#selectedTab=com.atlassian.jira.plugin.system.project%3Aroadmap-panel>
 
 =head1 SEE ALSO
@@ -535,7 +659,7 @@ Please see: L<https://logiclab.jira.com/browse/MBB#selectedTab=com.atlassian.jir
 
 =head1 MOTIVATION
 
-The motivation was driven by two things.
+The motivation for development of this module was driven by two things.
 
 =over
 
@@ -547,7 +671,7 @@ The motivation was driven by two things.
 
 I have a few perks and one of them is that I never get to automate stuff until
 very late and I always regret that. So when I released
-L<Bundle::JONASBN|Bundle::JONASBN>, now L<Task::BeLike::JONASBN::Task::BeLike::JONASBN>
+L<Bundle::JONASBN|Bundle::JONASBN>, now L<Task::BeLike::JONASBN|Task::BeLike::JONASBN>
 I thought I might aswell get it automated right away.
 
 This module lived for a long time as a part of L<Bundle::JONASBN|Bundle::JONASBN>
@@ -577,9 +701,8 @@ but then I needed it for some other distributions, so I decided to separate it o
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010-2013 jonasbn, all rights reserved.
+Copyright 2010-2015 jonasbn, all rights reserved.
 
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+Module::Build::Bundle is released under the Artistic License 2.0
 
 =cut
